///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      XPSRVCE.CPP
//
//  Description
//      This file implements the entry points, XPProviderInit and 
//      ServiceEntry for this provider's DLL.
//
//  Author
//      Irving De la Cruz
//
//  Revision: 1.0
//
// Written for Microsoft Windows Developer Support
// Copyright (c) 1994-1995 Microsoft Corporation. All rights reserved.
//
#include "XPCUST.H"

#define DO_INFO_TRACES
#ifdef DO_INFO_TRACES
#define InfoTrace(a)   TraceInfoMessage(a)
#else
#define InfoTrace(a)   
#endif // DO_INFO_TRACES


// Static data for the transport provider configuration property sheets
TCHAR szFilter[] = TEXT("*"); // "*" to allow any character

// Control description structures
DTBLPAGE  DtPage  = {sizeof(DTBLPAGE), fMapiUnicode};

DTBLLABEL DtLabel = {sizeof(DTBLLABEL), fMapiUnicode};

DTBLEDIT  DtCtlEmailAddr   = {sizeof(DTBLEDIT), fMapiUnicode, 31, PR_EMAIL_ADDRESS};
DTBLEDIT  DtCtlDisplayName = {sizeof(DTBLEDIT), fMapiUnicode, 31, PR_SKEL_USER_NAME};
DTBLEDIT  DtCtlAddressType = {sizeof(DTBLEDIT), fMapiUnicode, 15, PR_ADDRTYPE};

// Description table for the controls in the first page
DTCTL ConfigPage[] =
{
    {DTCT_PAGE,  0,           NULL, 0, NULL,     0,                &DtPage           },
    {DTCT_LABEL, 0,           NULL, 0, NULL,     IDC_STATIC1,      &DtLabel          },
    {DTCT_LABEL, 0,           NULL, 0, NULL,     IDC_STATIC2,      &DtLabel          },
    {DTCT_LABEL, 0,           NULL, 0, NULL,     IDC_STATIC3,      &DtLabel          },
    {DTCT_EDIT,  DT_REQUIRED, NULL, 0, szFilter, IDC_DISPLAYNAME,  &DtCtlDisplayName },
    {DTCT_EDIT,  DT_REQUIRED, NULL, 0, szFilter, IDC_EMAILADDRESS, &DtCtlEmailAddr   },
    {DTCT_EDIT,  DT_REQUIRED, NULL, 0, szFilter, IDC_ADDRESSTYPE,  &DtCtlAddressType }
};

#define NUM_CONFIGPAGE_CTLS     7

DTPAGE DtPropPage;

// Global pointer to the MAPI memory allocation functions
LPALLOCATEBUFFER    gpfnAllocateBuffer;  // MAPIAllocateBuffer function
LPALLOCATEMORE      gpfnAllocateMore;    // MAPIAllocateMore function    
LPFREEBUFFER        gpfnFreeBuffer;      // MAPIFreeBuffer function      

///////////////////////////////////////////////////////////////////////////////
//    ServiceEntry()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      Called by the profile setup administration tool to display the provider
//      configuration properties for this transport provider
//
//    Return Value
//      An HRESULT
//
HRESULT STDAPICALLTYPE ServiceEntry (HINSTANCE          hInstance,
                                     LPMALLOC           pMallocObj,
                                     LPMAPISUP          pSupObj,
                                     ULONG              ulUIParam,
                                     ULONG              ulFlags,
                                     ULONG              ulContext,
                                     ULONG              ulCfgPropCount,
                                     LPSPropValue       pCfgProps,
                                     LPPROVIDERADMIN    pAdminProvObj,
                                     LPMAPIERROR *      ppMAPIError)
{
    // Look at TRACES.H and TRACES.CPP for more options to the InitTraces() function
    // Send output only to the COM1 port
    InitTraces (0);
    // Send output to a console window BUT NOT to the COM1
    //InitTraces (TRACES_CONSOLE | TRACES_NO_COM_OUTPUT);
    // Send output to COM1 port AND a console window AND to a log file in C:\MAPILOG.TXT
    //InitTraces (TRACES_CONSOLE | TRACES_LOG_FILE);
    InfoTrace ("ServiceEntry function called");
    HRESULT hResult = S_OK;
    LPPROFSECT pProfileObj = NULL;
    LPSPropValue pProps = NULL;
    HANDLE hUIMutex = NULL;

    // What context where were we called in?
    if (MSG_SERVICE_INSTALL == ulContext ||
        MSG_SERVICE_UNINSTALL == ulContext ||
        MSG_SERVICE_DELETE == ulContext ||
        MSG_SERVICE_PROVIDER_CREATE == ulContext ||
        MSG_SERVICE_PROVIDER_DELETE == ulContext)
    {
        // We don't handle any of these contexts
        TraceString1 ("ServiceEntry: Immediate return. Context: %d", ulContext);
        goto ErrorExit;
    }
    TraceString1 ("ServiceEntry: Context for call: %d", ulContext);
    
    // Get the MAPI memory allocation routines we'll be needing
    hResult = pSupObj->GetMemAllocRoutines (&gpfnAllocateBuffer, &gpfnAllocateMore, &gpfnFreeBuffer);
    if (hResult)
    {
        TraceResult ("ServiceEntry: Failed to get the memory allocation functions", hResult);
        goto ErrorExit;
    }

    // Open the Profile Section for this Transport Provider
    hResult = OpenProviderProfileSection (pAdminProvObj, &pProfileObj);
    if (hResult)
    {
        goto ErrorExit;
    }
    
    // Profile is open, get the properties out of it
    ULONG  ulPropCount;
    hResult = pProfileObj->GetProps ((LPSPropTagArray)&sptLogonArray,
                                     fMapiUnicode,
                                     &ulPropCount,
                                     &pProps);
    if (FAILED(hResult))
    {
        TraceResult ("ServiceEntry: Failed to get profile section properties", hResult);
        goto ErrorExit;
    }
    ASSERT (NUM_LOGON_PROPERTIES == ulPropCount);
    if (pCfgProps && ulCfgPropCount)
    {
        MergeConfigProps (ulCfgPropCount, pCfgProps, pProps);
        hResult = pProfileObj->SetProps (ulPropCount, pProps, NULL);
        TraceResult ("ServiceEntry: Failed to save the properties back into the profile", hResult);
        if (!(UI_SERVICE & ulFlags))
        {
            goto ErrorExit;
        }
    }

    // If we can, display the UI...
    if (SERVICE_UI_ALWAYS & ulFlags || SERVICE_UI_ALLOWED & ulFlags)
    {
        hUIMutex = CreateMutex (NULL, FALSE, CONFIG_UI_MUTEX);
        if (NULL == hUIMutex)
        {
            TraceResult ("ServiceEntry: Failed to create UI mutext", GetLastError());
        }
        if (WAIT_TIMEOUT == WaitForSingleObject (hUIMutex, 250)) // Wait for the MUTEX for 1/4 second
        {
            PrivateMessageBox (IDS_MSG_CONFIG_UI_IN_USE,
                               (HWND)ulUIParam,
                               MB_OK | MB_ICONSTOP,
                               hInstance);
            hResult = MAPI_E_BUSY;
            goto ErrorExit;
        }
        
        // Fill in the logon UI structure
        XPDLG XPDialog = { 0 };
        XPDialog.hInst      = hInstance;
        XPDialog.hWnd       = (HWND)ulUIParam;
        XPDialog.ppProps    = &pProps;
        XPDialog.pMallocObj = pMallocObj;
        XPDialog.pSupObj    = pSupObj;
        XPDialog.ulFlags    = MSG_SERVICE_UI_READ_ONLY & ulFlags ? UI_READONLY : 0;
        hResult = DoLogonDlg (&XPDialog);
        if (hResult)
        {
            goto ErrorExit;
        }
    }
    hResult = pProfileObj->SetProps (ulPropCount, pProps, NULL);
    TraceResult ("ServiceEntry: Failed to save the properties back into the profile", hResult);

ErrorExit:
    if (pProfileObj)
    {
        pProfileObj->Release();
    }
    if (gpfnFreeBuffer)
    {
        gpfnFreeBuffer (pProps);
    }
    if (hUIMutex)
    {
        ReleaseMutex (hUIMutex);
        CloseHandle (hUIMutex);
    }
    // Close down the traces
    UnInitTraces();
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    OpenProviderProfileSection()
//
//    Parameters
//      pAdminProvObj   Pointer to an IProviderAdmin object
//      ppProfSectObj   Pointer to a pointer where this function returns a 
//                      IProfSect object open through the pAdmingProvObj object
//
//    Purpose
//      This function opens the profile section where this provider stores its
//      settings for the current profile
//
//    Return Value
//      An HRESULT
//
HRESULT WINAPI OpenProviderProfileSection (LPPROVIDERADMIN pAdminProvObj,
                                           LPPROFSECT *    ppProfSectObj)
{
    // Get the provider table where the settings are stored
    LPMAPITABLE pTableObj;
    HRESULT hResult = pAdminProvObj->GetProviderTable (0, &pTableObj);
    if (hResult)
    {
        TraceResult ("OpenProviderProfileSection: GetProviderTable failed", hResult);
        return hResult;
    }

    // Since we are going only looking for one property, limit the view
    // of the columns to what we are interested in
    static const SPropTagArray sptProvider = { 1, { PR_PROVIDER_UID } };
    LPSRowSet pRows;
    hResult = pTableObj->SetColumns ((LPSPropTagArray)&sptProvider, 0);
    TraceResult ("OpenProviderProfileSection: Failed to set the columns", hResult);
    if (!hResult)
    {
        // Query only one row, the user's row with the settings in it
        hResult = pTableObj->QueryRows (1, 0, &pRows);
        TraceResult ("OpenProviderProfileSection: Failed to query the rows", hResult);
        if (!hResult)
        {
            ASSERT (PR_PROVIDER_UID == pRows->aRow->lpProps->ulPropTag);
            hResult = pAdminProvObj->OpenProfileSection ((LPMAPIUID)pRows->aRow->lpProps->Value.bin.lpb,
                                                         NULL,
                                                         MAPI_MODIFY,
                                                         ppProfSectObj);
            FreeProws (pRows);
            if (hResult)
            {
                // Report failure, if any
                TraceResult ("OpenProviderProfileSection: OpenProfileSection failed", hResult);
                hResult = MAPI_E_NOT_FOUND;
            }
        }
    }
    pTableObj->Release();
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    MergeConfigProps()
//
//    Parameters
//      ulCfgProps      Number of properties for configuration
//      pCfgProps       Array of properties to configure the transport
//      pLogonProps     Array of properties found in the profile section of
//                      this transport
//
//    Purpose
//      This function merges a property array that the user (or configuration
//      application) passed into ServiceEntry with the property array of
//      Transport Logon stored in the profile. This merged array is what
//      either gets passed into the XP Logon dialog (if UI is allowed) and
//      gets written out to the profile section for the transport provider.
//
//    Return Value
//      None
//
void WINAPI MergeConfigProps (ULONG                  ulCfgProps,
                              LPSPropValue           pCfgProps,
                              LPSPropValue           pLogonProps)
{
    // Traverse the configuration property array
    while (ulCfgProps--)
    {
        // For the particular property found, set the proptag (in case
        // the one in the profile was not valid), and the value. The
        // value will be the value passed in the configuration prop array.
        switch (pCfgProps[ulCfgProps].ulPropTag)
        {
            case PR_SKEL_USER_NAME :
                pLogonProps[DISPLAY_NAME] = pCfgProps[ulCfgProps];
                break;
            case PR_EMAIL_ADDRESS :
                pLogonProps[EMAIL_ADDRESS] = pCfgProps[ulCfgProps];
                break;
            case PR_ADDRTYPE :
                pLogonProps[ADDRESS_TYPE] = pCfgProps[ulCfgProps];
                break;
            #ifdef _DEBUG
            default:
                // We should not get here
                TraceMessage ("MergeConfigProps: Bad property tag from for cfg");
                break;
            #endif // _DEBUG
        }
    }
}

///////////////////////////////////////////////////////////////////////////////
//    DoLogonDlg()
//
//    Parameters
//      pXPDialog   Pointer to an XPDialog structure.
//
//    Purpose
//      This functions gets called to display the configuration properties for
//      this transport. When invoked, it will parse the transport logon 
//      property array and initialize the fields in the property sheet with it.
//      The function could also get invoked to display the settings in 
//      READ-ONLY mode in which case we disable writing to the editable fields.
//
//    Return Value
//      An HRESULT
//
HRESULT WINAPI DoLogonDlg (PXPDLG pXPDialog)
{
    DWORD dwWait = WaitForSingleObject (pXPDialog->hUIMutex, 250); // Wait for the MUTEX for 1/4 second
    if (WAIT_TIMEOUT == dwWait)
    {
        PrivateMessageBox (IDS_MSG_CONFIG_UI_IN_USE,
                           pXPDialog->hWnd,
                           MB_OK | MB_ICONSTOP,
                           pXPDialog->hInst);
        return MAPI_E_BUSY;
    }

    LPMAPITABLE pTableObj    = NULL;
    LPPROPDATA  pPropDataObj = NULL;
    ULONG       ulPropCount;
    TCHAR       szDialogTitle[64];
    
    // Take ownership of the property value array, so calling code
    // will not leak memory
    LPSPropValue pProps = *pXPDialog->ppProps;
    *pXPDialog->ppProps = NULL;
    
    // Retrieve the dialog title string
    LoadString (pXPDialog->hInst, 
                IDS_CONFIG_TITLE,
                szDialogTitle,
                sizeof(szDialogTitle) - 1);
    
    // Now make a temporary property interface and feed it the data    
    HRESULT hResult = CreateIProp (&IID_IMAPIPropData,
                                   gpfnAllocateBuffer,
                                   gpfnAllocateMore,
                                   gpfnFreeBuffer,
                                   pXPDialog->pMallocObj,
                                   &pPropDataObj);
    if (hResult)
    {   
        TraceResult ("DoLogonDlg: CreateIProp failed to create IPropData object", hResult);
        goto ErrorExit;
    }
    
    // Set the properties in the property array. The properties MUST exist in
    // the array before invoking the property page with MAPI methods, 
    // otherwise the call fails
    hResult = pPropDataObj->SetProps (NUM_LOGON_PROPERTIES, pProps, NULL);    
    if (hResult)
    {
        TraceResult ("DoLogonDlg: The logon properties SetProps failed", hResult);
        goto ErrorExit;
    }

    DtPropPage.cctl = NUM_CONFIGPAGE_CTLS;   // Count of controls in the ConfigPage array
    DtPropPage.lpszComponent = TEXT("");
    DtPropPage.lpszResourceName = MAKEINTRESOURCE (IDD_CONFIGPAGE);
    DtPropPage.lpctl = ConfigPage;

    // If the UI_READONLY flag is set, then we are being called from the 
    // SettingDialog method in Read Only mode. Disable all of the controls
    // on the property sheets, else Enable them
    if (pXPDialog->ulFlags & UI_READONLY)
    {
        // Disable the controls (make them read only)
        DtPropPage.lpctl[4].ulCtlFlags &= ~DT_EDITABLE;
        DtPropPage.lpctl[5].ulCtlFlags &= ~DT_EDITABLE;
        DtPropPage.lpctl[6].ulCtlFlags &= ~DT_EDITABLE;
    }
    else
    {
        // Enable the controls (make them read/write)
        DtPropPage.lpctl[4].ulCtlFlags |= DT_EDITABLE;
        DtPropPage.lpctl[5].ulCtlFlags |= DT_EDITABLE;
        DtPropPage.lpctl[6].ulCtlFlags |= DT_EDITABLE;
    }
        
    // Create the display table for the logon dialog. It is based on dialog
    // resources plus the static information at the head of this module.       
    hResult = BuildDisplayTable (gpfnAllocateBuffer,
                                 gpfnAllocateMore,
                                 gpfnFreeBuffer,
                                 pXPDialog->pMallocObj,
                                 pXPDialog->hInst,
                                 1,
                                 &DtPropPage,
                                 fMapiUnicode,
                                 &pTableObj,
                                 NULL);        
    if (hResult)
    {
        TraceResult ("DoLogonDlg: BuildDisplayTable failed", hResult);
        goto ErrorExit;
    }

    // Display the dialog/property sheets
    hResult = pXPDialog->pSupObj->DoConfigPropsheet ((ULONG)pXPDialog->hWnd,
                                                     0,
                                                     szDialogTitle,
                                                     pTableObj,
                                                     (LPMAPIPROP)pPropDataObj,
                                                     0); // Default index for the initial property sheet
    if (hResult)
    {
        if (MAPI_E_USER_CANCEL != hResult)
        {
            TraceResult ("DoLogonDlg: DoConfigPropsheet failed", hResult);
        }
        goto ErrorExit;
    }

    // Retrieve the altered data
    gpfnFreeBuffer (pProps);
    pProps = NULL;
    hResult = pPropDataObj->GetProps ((LPSPropTagArray)&sptLogonArray,
                                      fMapiUnicode,
                                      &ulPropCount,
                                      &pProps);
    if (FAILED(hResult))
    {
        TraceResult ("DoLogonDlg: GetProps failed", hResult);
        goto ErrorExit;
    }
        
    // Return the new property array
    *pXPDialog->ppProps = pProps;
    pProps = NULL;      // do not free this in cleanup code below
    hResult = S_OK;

ErrorExit:
    // Release objects
    if (pTableObj)
    {
        pTableObj->Release();
    }
    if (pPropDataObj)
    {
        pPropDataObj->Release();
    }    
    gpfnFreeBuffer (pProps);
    ReleaseMutex (pXPDialog->hUIMutex);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    OptionDataCallBack()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      None. This function is just a place holder. Since we didn't register
//      any per-message or per-recipient options in IXPLogon::RegisterOptions,
//      this function will never get called.
//
//    Return Value
//      E_FAIL        Always
//
STDINITMETHODIMP_ (SCODE) OptionDataCallBack (LPXPLOGON     pXPLogon,
                                              HINSTANCE     hInstance,
                                              ULONG         ulFlags,
                                              ULONG         cbOptionData,
                                              LPBYTE        pbOptionData,
                                              LPMAPISUP     pMAPISup,
                                              LPMAPIPROP    pDataSource,
                                              LPMAPIPROP *  ppWrappedSource,
                                              LPTSTR *      ppszErrorMsg,
                                              LPTSTR *      ppszErrorComponent,
                                              ULONG *       pulErrorContext)
{
    InfoTrace ("OptionDataCallBack function called");
    return E_FAIL;
}

///////////////////////////////////////////////////////////////////////////////
//    GetMAPIError()
//
//    Parameters
//      ppMAPIError             Pointer to a location where the newly
//                              allocated MAPIERROR structure is returened. 
//      ulFlags                 Flags for the allocation of the structure
//      hError                  Handle to the error for which a string will
//                              be returned
//      hInstance               Handle to instance of DLL where the resource
//                              strings are located
//
//    Purpose
//      Return a MAPIERROR structure filled with the description of the
//      error passed in. The caller is responsible for releasing the memory
//      allocated here.
//
//    Return Value
//      An HRESULT
//
HRESULT WINAPI GetMAPIError (LPMAPIERROR *          ppMAPIError,
                             ULONG                  ulFlags,
                             HRESULT                hError,
                             HINSTANCE              hInstance)
{
    // The following array maps a string identifier (IDS) to an hResult
    // The order of HRESULTs in the array has an external dependency:
    // the order of elements in the array is dictated by the IDS definitions
    // in RESOURCE.H. This implicit association must be maintained for the 
    // strings associated with string identifiers to make sense. Thus, if 
    // either this structure or the RESOURCE.H defines changes, the other 
    // must change to match it
    static const HRESULT MapOfhResults[] =
    {
    	S_OK,

    	E_FAIL,
    	E_OUTOFMEMORY,
    	E_INVALIDARG,
    	E_NOINTERFACE,
    	E_ACCESSDENIED,

        MAPI_E_BUSY,
    	MAPI_E_NO_SUPPORT,
    	MAPI_E_NOT_FOUND,
    	MAPI_E_UNKNOWN_FLAGS,
    	MAPI_E_VERSION,
    	MAPI_E_USER_CANCEL,
        MAPI_E_BAD_CHARWIDTH,
        MAPI_E_UNCONFIGURED,
        
        MAPI_W_ERRORS_RETURNED
    };

    
    // Default the a return value in case we don't have the specified error
    // Allocate the memory for the MAPIERROR structure. The caller is
    // responsible for free this
    HRESULT hResult = gpfnAllocateBuffer (sizeof(MAPIERROR), (LPVOID *)ppMAPIError);
    if (hResult)
    {
        TraceResult ("GetMAPIError: Failed to allocate memory for the MAPIERROR structure", hResult);
        return hResult;
    }

    // Zero-out all the members in the structure
    ZeroMemory (*ppMAPIError, sizeof(MAPIERROR));
    (*ppMAPIError)->ulVersion = MAPI_ERROR_VERSION;
    // See if we have the string for the error. If so, make a copy 
    // and pass it back. The memory for the string will be linked to
    // the memory of the structure so that the caller can make a
    // single release on the structure
    
	// Search for the hResult in the HRESULT map
    UINT i, uMaxNum, uStringID;
    uStringID = 0;
    uMaxNum = sizeof (MapOfhResults) / sizeof (HRESULT);
    for (i=0; i<uMaxNum; i++ )
	{
		if (MapOfhResults[i] == hError)
		{
            break;
		}
	}

    if (i == uMaxNum)
    {
        uStringID = IDS_E_UNKNOWN_ERROR;
    }
	else
    {
        uStringID = i + IDS_SUCCESS;
    }
    
    TCHAR szBuffer[512];
	int nCharsRead = LoadString (hInstance, uStringID, szBuffer, sizeof(szBuffer) -1);
    if (!nCharsRead)
    {
        TraceString1 ("GetMAPIError: Could not find error string for ID %d", uStringID);
        hResult = MAPI_E_NOT_FOUND;
    }
    else
    {
    	// Allocate memory for return variable and set it.
        // The memory for this string is linked to the memory in the pMemBlock parameter
    	hResult = gpfnAllocateMore (Cbtszsize (szBuffer),
                                    *ppMAPIError,
    	                            (LPVOID *)&(*ppMAPIError)->lpszError);
    	if (!hResult)
        {
            lstrcpy ((*ppMAPIError)->lpszError, szBuffer);
            hResult = gpfnAllocateMore (Cbtszsize (TRANSPORT_DISPLAY_NAME_STRING),
                                        *ppMAPIError,
    	                                (LPVOID *)&(*ppMAPIError)->lpszComponent);
            if (!hResult)
            {
                lstrcpy ((*ppMAPIError)->lpszComponent, TRANSPORT_DISPLAY_NAME_STRING);
            }
            else
            {
                TraceResult ("GetMAPIError: Failed to allocate the memory for the component string", hResult);
            }
        }
        else
    	{
            TraceResult ("GetMAPIError: Failed to allocate the memory for the error string", hResult);
    	}
    }

    if (hResult)
    {
        gpfnFreeBuffer (*ppMAPIError);
        *ppMAPIError = NULL;
    }
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    PrivateMessageBox()
//
//    Parameters
//      ids             String ID of a string in the resource string table
//      hOwnerWnd       Window who will own the Message Box
//      uFlags          Flags (options) for the message box
//      
//    Purpose
//      This function display a message box with a string loaded from the
//      string table of this app. The ID of the string is passed in the ids
//      parameter. The dialog box is modal with respect to the window
//      identified in hOwnerWnd. The options to display the dialog box are
//      passed in uFlags
//      
//    Return Value
//      ID of the button pressed in the message box dialog box
//      
int WINAPI PrivateMessageBox (UINT          ids,
                              HWND          hOwnerWnd,
                              UINT          uFlags,
                              HINSTANCE     hInst)
{
    int nResponse = -1; // Default response
    ASSERT (hInst);
    TCHAR szBuffer[256];
    if (LoadString (hInst, ids, szBuffer, 255))
    {
        if (!(MB_ICONSTOP & uFlags))
        {
            uFlags |= MB_ICONINFORMATION;
        }
        // Show the message box and get the ID of the button pressed
        nResponse = MessageBox (hOwnerWnd,
                                szBuffer,
                                TEXT("PSS Skelatal TNEF Transport"),
                                uFlags | MB_SETFOREGROUND | MB_SYSTEMMODAL);
    }
    return nResponse;
}

///////////////////////////////////////////////////////////////////////////////
//    XPProviderInit()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      Entry pointe called by the MAPI spooler when a profile uses this 
//      transport. The spooler calls this method and expects a pointer to an
//      implementation of the IXPProvider interface. MAPI uses the returned
//      IXPProvider interface pointer to logon on the transport provider
//
//    Return Value
//      An HRESULT
//
STDINITMETHODIMP XPProviderInit (HINSTANCE          hInstance, 
                                 LPMALLOC           pMallocObj,
                                 LPALLOCATEBUFFER   pfnAllocateBuffer,
                                 LPALLOCATEMORE     pfnAllocateMore,
                                 LPFREEBUFFER       pfnFreeBuffer,
                                 ULONG              ulFlags,
                                 ULONG              ulMAPIVer,
                                 ULONG *            pulProviderVer,
                                 LPXPPROVIDER *     ppXPProviderObj)
{
    // Look at TRACES.H and TRACES.CPP for more options to the InitTraces() function
    InitTraces (0);                                       // Send output only to the COM1 port
    //InitTraces (TRACES_CONSOLE | TRACES_NO_COM_OUTPUT);     // Send output to a console window BUT NOT to the COM1
    //InitTraces (TRACES_CONSOLE | TRACES_LOG_FILE);        // Send output to COM1 port AND a console window AND to a log file in C:\MAPILOG.TXT
    InfoTrace ("XPProviderInit function called");
    *pulProviderVer = CURRENT_SPI_VERSION;
    if (ulMAPIVer < CURRENT_SPI_VERSION)
    {
        TraceMessage ("XPProviderInit: The version of the subsystem cannot handle this version of the provider");
        return MAPI_E_VERSION;
    }
    HRESULT hResult = S_OK;
    // Save the pointer to the allocation routines in global variables
    gpfnAllocateBuffer = pfnAllocateBuffer;
    gpfnAllocateMore   = pfnAllocateMore;
    gpfnFreeBuffer     = pfnFreeBuffer;
    // Allocate a new XPProvider object, the constructor will initialize the data member
    CXPProvider * pXPProvider = new CXPProvider (hInstance, pMallocObj);
    if (!pXPProvider)
    {
        TraceMessage ("XPProviderInit: Failed to allocate new CXPProvider object failed");
        hResult = E_OUTOFMEMORY;
    }
    // Copy pointer to the allocated object back into the return IXPProvider object pointer
    *ppXPProviderObj = (LPXPPROVIDER)pXPProvider;        
    return hResult;
}

// End of file for XPSRVCE.CPP
